"use strict";Object.defineProperty(exports, "__esModule", {value: true});// src/utils.ts
function getBodyJson(req) {
  return new Promise((resolve, reject) => {
    let body = "";
    req.on("data", (chunk) => body += chunk);
    req.on("error", reject);
    req.on("end", () => {
      try {
        resolve(JSON.parse(body) || {});
      } catch (e) {
        reject(e);
      }
    });
  });
}
function get(obj, key) {
  return key.split(".").reduce((acc, key2) => acc == null ? void 0 : acc[key2], obj);
}
function set(obj, key, value) {
  const keys = key.split(".");
  let acc = obj;
  keys.slice(0, -1).forEach((key2) => {
    if (!acc[key2])
      acc[key2] = {};
    acc = acc[key2];
  });
  acc[keys[keys.length - 1]] = value;
}

// src/index.ts
var URL_PREFIX = "/@server-ref/";
var VIRTUAL_PREFIX = "server-ref:";
var WS_EVENT = "server-ref";
function VitePluginServerRef(options = {}) {
  const {
    state = {},
    debounce = 10,
    debug = false,
    defaultValue = () => void 0,
    clientVue = "vue"
  } = options;
  return {
    name: "vite-plugin-vue-server-ref",
    resolveId(id) {
      if (id.startsWith(VIRTUAL_PREFIX))
        return URL_PREFIX + id.slice(VIRTUAL_PREFIX.length);
      return id.startsWith(URL_PREFIX) ? id : null;
    },
    configureServer(server) {
      server.middlewares.use(async (req, res, next) => {
        var _a, _b;
        if (!((_a = req.url) == null ? void 0 : _a.startsWith(URL_PREFIX)) || req.method !== "POST")
          return next();
        const key = req.url.slice(URL_PREFIX.length);
        const { data, timestamp, patch, source } = await getBodyJson(req);
        const module = server.moduleGraph.getModuleById(URL_PREFIX + key);
        if (module)
          server.moduleGraph.invalidateModule(module);
        if (patch)
          set(state, key, Object.assign(get(state, key), patch));
        else
          set(state, key, data);
        server.ws.send({
          type: "custom",
          event: WS_EVENT,
          data: {
            source,
            key,
            data,
            patch,
            timestamp
          }
        });
        (_b = options.onChanged) == null ? void 0 : _b.call(options, key, data, patch, timestamp);
        res.write("");
        res.end();
      });
    },
    load(id) {
      var _a;
      if (!id.startsWith(URL_PREFIX))
        return null;
      const key = id.slice(URL_PREFIX.length).replace(/\//g, ".");
      return `
import { ref, watch } from "${clientVue}"

const data = ref(${JSON.stringify((_a = get(state, key)) != null ? _a : defaultValue(key))})

data.syncUp = true
data.syncDown = true
data.paused = false
data.patch = async () => false
data.onSet = () => {}
data.onPatch = () => {}

if (import.meta.hot) {
  function uid() {
    return Math.random().toString(36).replace(/[^a-z]+/g, '').substr(2, 10);
  }

  function post(payload) {
    ${debug ? `console.log("[server-ref] [${key}] outgoing", payload)` : ""}
    return fetch('${URL_PREFIX + key}', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
      },
      body: JSON.stringify(payload)
    })
  }

  ${debug ? `console.log("[server-ref] [${key}] ref", data)` : ""}
  ${debug ? `console.log("[server-ref] [${key}] initial", data.value)` : ""}

  const id = uid()
  let skipWatch = false
  let timer = null
  import.meta.hot.on("${WS_EVENT}", (payload) => {
    if (!data.syncDown || data.paused || payload.key !== "${key}" || payload.source === id)
      return
    skipWatch = true
    if (payload.patch) {
      data.onPatch(payload.patch)
      data.value = Object.assign(data.value, payload.patch)
      ${debug ? `console.log("[server-ref] [${key}] patch incoming", payload.patch)` : ""}
    }
    else {
      data.onSet(payload.data)
      data.value = payload.data
      ${debug ? `console.log("[server-ref] [${key}] incoming", payload.data)` : ""}
    }
    skipWatch = false
  })

  data.patch = async (patch) => {
    if (!data.syncUp || data.paused)
      return false
    data.value = Object.assign(data.value, patch)
    return post({
      source: id,
      patch,
      timestamp: Date.now(),
    })
  }

  watch(data, (v) => {
    if (timer)
      clearTimeout(timer)
    if (!data.syncUp || data.paused || skipWatch)
      return

    timer = setTimeout(()=>{
      post({
        source: id,
        data: data.value,
        timestamp: Date.now(),
      })
    }, ${debounce})
  }, { flush: 'sync', deep: true })
}

export default data
`;
    }
  };
}


exports.default = VitePluginServerRef;
